/* SerializableDouble.c generated by valac 0.39.3.9-ecb8a, the Vala compiler
 * generated from SerializableDouble.vala, do not modify */

/* -*- Mode: vala; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/* SerializableInt.vala
 *
 * Copyright (C) 2015  Daniel Espinosa <esodan@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Daniel Espinosa <esodan@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gio/gio.h>
#include <float.h>
#include <math.h>


#define GXML_TYPE_SERIALIZABLE_PROPERTY (gxml_serializable_property_get_type ())
#define GXML_SERIALIZABLE_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_PROPERTY, GXmlSerializableProperty))
#define GXML_IS_SERIALIZABLE_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_PROPERTY))
#define GXML_SERIALIZABLE_PROPERTY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_SERIALIZABLE_PROPERTY, GXmlSerializablePropertyIface))

typedef struct _GXmlSerializableProperty GXmlSerializableProperty;
typedef struct _GXmlSerializablePropertyIface GXmlSerializablePropertyIface;

#define GXML_TYPE_NODE (gxml_node_get_type ())
#define GXML_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NODE, GXmlNode))
#define GXML_IS_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NODE))
#define GXML_NODE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NODE, GXmlNodeIface))

typedef struct _GXmlNode GXmlNode;
typedef struct _GXmlNodeIface GXmlNodeIface;

#define GXML_TYPE_ELEMENT_LIST (gxml_element_list_get_type ())
#define GXML_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementList))
#define GXML_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))
#define GXML_IS_ELEMENT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_ELEMENT_LIST))
#define GXML_IS_ELEMENT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_ELEMENT_LIST))
#define GXML_ELEMENT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_ELEMENT_LIST, GXmlElementListClass))

typedef struct _GXmlElementList GXmlElementList;
typedef struct _GXmlElementListClass GXmlElementListClass;

#define GXML_TYPE_NAMESPACE (gxml_namespace_get_type ())
#define GXML_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_NAMESPACE, GXmlNamespace))
#define GXML_IS_NAMESPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_NAMESPACE))
#define GXML_NAMESPACE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_NAMESPACE, GXmlNamespaceIface))

typedef struct _GXmlNamespace GXmlNamespace;
typedef struct _GXmlNamespaceIface GXmlNamespaceIface;

#define GXML_TYPE_NODE_TYPE (gxml_node_type_get_type ())

#define GXML_TYPE_DOCUMENT (gxml_document_get_type ())
#define GXML_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_DOCUMENT, GXmlDocument))
#define GXML_IS_DOCUMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_DOCUMENT))
#define GXML_DOCUMENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GXML_TYPE_DOCUMENT, GXmlDocumentIface))

typedef struct _GXmlDocument GXmlDocument;
typedef struct _GXmlDocumentIface GXmlDocumentIface;

#define GXML_TYPE_SERIALIZABLE_DOUBLE (gxml_serializable_double_get_type ())
#define GXML_SERIALIZABLE_DOUBLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDouble))
#define GXML_SERIALIZABLE_DOUBLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDoubleClass))
#define GXML_IS_SERIALIZABLE_DOUBLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GXML_TYPE_SERIALIZABLE_DOUBLE))
#define GXML_IS_SERIALIZABLE_DOUBLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GXML_TYPE_SERIALIZABLE_DOUBLE))
#define GXML_SERIALIZABLE_DOUBLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDoubleClass))

typedef struct _GXmlSerializableDouble GXmlSerializableDouble;
typedef struct _GXmlSerializableDoubleClass GXmlSerializableDoubleClass;
typedef struct _GXmlSerializableDoublePrivate GXmlSerializableDoublePrivate;
enum  {
	GXML_SERIALIZABLE_DOUBLE_0_PROPERTY,
	GXML_SERIALIZABLE_DOUBLE_NUM_PROPERTIES
};
static GParamSpec* gxml_serializable_double_properties[GXML_SERIALIZABLE_DOUBLE_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))

struct _GXmlNamespaceIface {
	GTypeInterface parent_iface;
	gchar* (*get_uri) (GXmlNamespace* self);
	gchar* (*get_prefix) (GXmlNamespace* self);
};

typedef enum  {
	GXML_NODE_TYPE_INVALID = 0,
	GXML_NODE_TYPE_ELEMENT = 1,
	GXML_NODE_TYPE_ATTRIBUTE,
	GXML_NODE_TYPE_TEXT,
	GXML_NODE_TYPE_CDATA_SECTION,
	GXML_NODE_TYPE_ENTITY_REFERENCE,
	GXML_NODE_TYPE_ENTITY,
	GXML_NODE_TYPE_PROCESSING_INSTRUCTION,
	GXML_NODE_TYPE_COMMENT,
	GXML_NODE_TYPE_DOCUMENT,
	GXML_NODE_TYPE_DOCUMENT_TYPE,
	GXML_NODE_TYPE_DOCUMENT_FRAGMENT,
	GXML_NODE_TYPE_NOTATION
} GXmlNodeType;

struct _GXmlDocumentIface {
	GTypeInterface parent_iface;
	GXmlNode* (*create_element) (GXmlDocument* self, const gchar* name, GError** error);
	GXmlNode* (*create_text) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_comment) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_cdata) (GXmlDocument* self, const gchar* text);
	GXmlNode* (*create_pi) (GXmlDocument* self, const gchar* target, const gchar* data);
	gboolean (*save) (GXmlDocument* self, GCancellable* cancellable, GError** error);
	gboolean (*save_as) (GXmlDocument* self, GFile* f, GCancellable* cancellable, GError** error);
	gboolean (*get_indent) (GXmlDocument* self);
	void (*set_indent) (GXmlDocument* self, gboolean value);
	gboolean (*get_ns_top) (GXmlDocument* self);
	void (*set_ns_top) (GXmlDocument* self, gboolean value);
	gboolean (*get_prefix_default_ns) (GXmlDocument* self);
	void (*set_prefix_default_ns) (GXmlDocument* self, gboolean value);
	gboolean (*get_backup) (GXmlDocument* self);
	void (*set_backup) (GXmlDocument* self, gboolean value);
	GXmlNode* (*get_root) (GXmlDocument* self);
	GFile* (*get_file) (GXmlDocument* self);
	void (*set_file) (GXmlDocument* self, GFile* value);
};

struct _GXmlNodeIface {
	GTypeInterface parent_iface;
	GXmlNode* (*get) (GXmlNode* self, const gchar* key);
	GXmlElementList* (*get_elements_by_property_value) (GXmlNode* self, const gchar* property, const gchar* value);
	GXmlElementList* (*get_elements_by_name) (GXmlNode* self, const gchar* name);
	GXmlElementList* (*get_elements_by_name_ns) (GXmlNode* self, const gchar* name, const gchar* ns);
	gchar* (*to_string) (GXmlNode* self);
	gboolean (*set_namespace) (GXmlNode* self, const gchar* uri, const gchar* prefix);
	gchar* (*ns_prefix) (GXmlNode* self);
	gchar* (*ns_uri) (GXmlNode* self);
	GeeList* (*get_namespaces) (GXmlNode* self);
	GeeBidirList* (*get_children_nodes) (GXmlNode* self);
	GeeMap* (*get_attrs) (GXmlNode* self);
	gchar* (*get_name) (GXmlNode* self);
	gchar* (*get_value) (GXmlNode* self);
	void (*set_value) (GXmlNode* self, const gchar* value);
	GXmlNodeType (*get_type_node) (GXmlNode* self);
	GXmlDocument* (*get_document) (GXmlNode* self);
	GXmlNode* (*get_parent) (GXmlNode* self);
};

struct _GXmlSerializablePropertyIface {
	GTypeInterface parent_iface;
	gchar* (*get_serializable_property_value) (GXmlSerializableProperty* self);
	void (*set_serializable_property_value) (GXmlSerializableProperty* self, const gchar* val);
	gboolean (*deserialize_property) (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
	GXmlNode* (*serialize_property) (GXmlSerializableProperty* self, GXmlNode* property_node, GParamSpec* prop, gboolean nick, GError** error);
};

struct _GXmlSerializableDouble {
	GObject parent_instance;
	GXmlSerializableDoublePrivate * priv;
	gchar* _val;
	gint _fraction;
};

struct _GXmlSerializableDoubleClass {
	GObjectClass parent_class;
};


static gpointer gxml_serializable_double_parent_class = NULL;
static GXmlSerializablePropertyIface * gxml_serializable_double_gxml_serializable_property_parent_iface = NULL;

GType gxml_element_list_get_type (void) G_GNUC_CONST;
GType gxml_namespace_get_type (void) G_GNUC_CONST;
GType gxml_node_type_get_type (void) G_GNUC_CONST;
GType gxml_document_get_type (void) G_GNUC_CONST;
GType gxml_node_get_type (void) G_GNUC_CONST;
GType gxml_serializable_property_get_type (void) G_GNUC_CONST;
GType gxml_serializable_double_get_type (void) G_GNUC_CONST;
gint gxml_serializable_double_get_fraction (GXmlSerializableDouble* self);
void gxml_serializable_double_set_fraction (GXmlSerializableDouble* self, 
                                            gint fraction);
gdouble gxml_serializable_double_get_value (GXmlSerializableDouble* self);
void gxml_serializable_double_set_value (GXmlSerializableDouble* self, 
                                         gdouble val);
static gchar* gxml_serializable_double_real_get_serializable_property_value (GXmlSerializableProperty* base);
static void gxml_serializable_double_real_set_serializable_property_value (GXmlSerializableProperty* base, 
                                                                    const gchar* val);
gchar* gxml_serializable_double_to_string (GXmlSerializableDouble* self);
gchar* gxml_serializable_double_format (GXmlSerializableDouble* self, 
                                        const gchar* f);
GXmlSerializableDouble* gxml_serializable_double_new (void);
GXmlSerializableDouble* gxml_serializable_double_construct (GType object_type);
static GObject * gxml_serializable_double_constructor (GType type, 
                                                guint n_construct_properties, 
                                                GObjectConstructParam * construct_properties);
void gxml_init_init (void);
static void gxml_serializable_double_finalize (GObject * obj);


gint
gxml_serializable_double_get_fraction (GXmlSerializableDouble* self)
{
	gint result = 0;
	gint _tmp0_;
	g_return_val_if_fail (self != NULL, 0);
	_tmp0_ = self->_fraction;
	result = _tmp0_;
	return result;
}


void
gxml_serializable_double_set_fraction (GXmlSerializableDouble* self, 
                                       gint fraction)
{
	gint v = 0;
	gint _tmp0_;
	gint _tmp1_;
	g_return_if_fail (self != NULL);
	v = fraction;
	_tmp0_ = v;
	if (_tmp0_ < 0) {
		self->_fraction = -1;
	}
	_tmp1_ = v;
	self->_fraction = _tmp1_;
}


static gdouble
double_parse (const gchar* str)
{
	gdouble result = 0.0;
	gdouble _tmp0_;
	g_return_val_if_fail (str != NULL, 0.0);
	_tmp0_ = g_ascii_strtod (str, NULL);
	result = _tmp0_;
	return result;
}


gdouble
gxml_serializable_double_get_value (GXmlSerializableDouble* self)
{
	gdouble result = 0.0;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, 0.0);
	_tmp0_ = self->_val;
	result = double_parse (_tmp0_);
	return result;
}


static gchar*
double_to_string (gdouble self)
{
	gchar* result = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gint _tmp1__length1;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	_tmp0_ = g_new0 (gchar, G_ASCII_DTOSTR_BUF_SIZE);
	_tmp1_ = _tmp0_;
	_tmp1__length1 = G_ASCII_DTOSTR_BUF_SIZE;
	_tmp2_ = g_ascii_dtostr (_tmp1_, G_ASCII_DTOSTR_BUF_SIZE, self);
	_tmp3_ = g_strdup (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp1_ = (g_free (_tmp1_), NULL);
	result = _tmp4_;
	return result;
}


void
gxml_serializable_double_set_value (GXmlSerializableDouble* self, 
                                    gdouble val)
{
	gchar* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = double_to_string (val);
	_g_free0 (self->_val);
	self->_val = _tmp0_;
}


static gchar*
gxml_serializable_double_real_get_serializable_property_value (GXmlSerializableProperty* base)
{
	GXmlSerializableDouble * self;
	gchar* result = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	self = (GXmlSerializableDouble*) base;
	_tmp0_ = self->_val;
	_tmp1_ = g_strdup (_tmp0_);
	result = _tmp1_;
	return result;
}


static void
gxml_serializable_double_real_set_serializable_property_value (GXmlSerializableProperty* base, 
                                                               const gchar* val)
{
	GXmlSerializableDouble * self;
	gchar* _tmp0_;
	self = (GXmlSerializableDouble*) base;
	_tmp0_ = g_strdup (val);
	_g_free0 (self->_val);
	self->_val = _tmp0_;
}


static const gchar*
string_to_string (const gchar* self)
{
	const gchar* result = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	result = self;
	return result;
}


gchar*
gxml_serializable_double_to_string (GXmlSerializableDouble* self)
{
	gchar* result = NULL;
	gchar* f = NULL;
	gint _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	const gchar* _tmp7_;
	gchar* _tmp15_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->_fraction;
	_tmp1_ = g_strdup_printf ("%i", _tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = g_strconcat ("%.", _tmp2_, NULL);
	_tmp4_ = _tmp3_;
	_tmp5_ = g_strconcat (_tmp4_, "f", NULL);
	_tmp6_ = _tmp5_;
	_g_free0 (_tmp4_);
	_g_free0 (_tmp2_);
	f = _tmp6_;
	_tmp7_ = self->_val;
	if (_tmp7_ != NULL) {
		const gchar* _tmp8_;
		const gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		const gchar* _tmp12_;
		gchar* _tmp13_;
		gchar* _tmp14_;
		_tmp8_ = f;
		_tmp9_ = self->_val;
		_tmp10_ = g_strdup_printf (_tmp8_, double_parse (_tmp9_));
		_tmp11_ = _tmp10_;
		_tmp12_ = string_to_string (_tmp11_);
		_tmp13_ = g_strdup (_tmp12_);
		_tmp14_ = _tmp13_;
		_g_free0 (_tmp11_);
		result = _tmp14_;
		_g_free0 (f);
		return result;
	}
	_tmp15_ = g_strdup ("");
	result = _tmp15_;
	_g_free0 (f);
	return result;
}


gchar*
gxml_serializable_double_format (GXmlSerializableDouble* self, 
                                 const gchar* f)
{
	gchar* result = NULL;
	const gchar* _tmp0_;
	gchar* _tmp3_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (f != NULL, NULL);
	_tmp0_ = self->_val;
	if (_tmp0_ != NULL) {
		const gchar* _tmp1_;
		gchar* _tmp2_;
		_tmp1_ = self->_val;
		_tmp2_ = g_strdup_printf (f, double_parse (_tmp1_));
		result = _tmp2_;
		return result;
	}
	_tmp3_ = g_strdup ("");
	result = _tmp3_;
	return result;
}


GXmlSerializableDouble*
gxml_serializable_double_construct (GType object_type)
{
	GXmlSerializableDouble * self = NULL;
	self = (GXmlSerializableDouble*) g_object_new (object_type, NULL);
	return self;
}


GXmlSerializableDouble*
gxml_serializable_double_new (void)
{
	return gxml_serializable_double_construct (GXML_TYPE_SERIALIZABLE_DOUBLE);
}


static GObject *
gxml_serializable_double_constructor (GType type, 
                                      guint n_construct_properties, 
                                      GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	GXmlSerializableDouble * self;
	parent_class = G_OBJECT_CLASS (gxml_serializable_double_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDouble);
	gxml_init_init ();
	return obj;
}


static void
gxml_serializable_double_class_init (GXmlSerializableDoubleClass * klass)
{
	gxml_serializable_double_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->constructor = gxml_serializable_double_constructor;
	G_OBJECT_CLASS (klass)->finalize = gxml_serializable_double_finalize;
}


static void
gxml_serializable_double_gxml_serializable_property_interface_init (GXmlSerializablePropertyIface * iface)
{
	gxml_serializable_double_gxml_serializable_property_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_serializable_property_value = (gchar* (*) (GXmlSerializableProperty *)) gxml_serializable_double_real_get_serializable_property_value;
	iface->set_serializable_property_value = (void (*) (GXmlSerializableProperty *, const gchar*)) gxml_serializable_double_real_set_serializable_property_value;
}


static void
gxml_serializable_double_instance_init (GXmlSerializableDouble * self)
{
	self->_val = NULL;
	self->_fraction = -1;
}


static void
gxml_serializable_double_finalize (GObject * obj)
{
	GXmlSerializableDouble * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GXML_TYPE_SERIALIZABLE_DOUBLE, GXmlSerializableDouble);
	_g_free0 (self->_val);
	G_OBJECT_CLASS (gxml_serializable_double_parent_class)->finalize (obj);
}


/**
 * Represent any real double value property to be added as a
 * {@link GXml.Attribute} to a {@link GXml.Element}
 *
 * This object can be used when the property could be removed from
 * serialization, specially if missing on XML have some
 * missing and avoidable.
 */
GType
gxml_serializable_double_get_type (void)
{
	static volatile gsize gxml_serializable_double_type_id__volatile = 0;
	if (g_once_init_enter (&gxml_serializable_double_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GXmlSerializableDoubleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gxml_serializable_double_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GXmlSerializableDouble), 0, (GInstanceInitFunc) gxml_serializable_double_instance_init, NULL };
		static const GInterfaceInfo gxml_serializable_property_info = { (GInterfaceInitFunc) gxml_serializable_double_gxml_serializable_property_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gxml_serializable_double_type_id;
		gxml_serializable_double_type_id = g_type_register_static (G_TYPE_OBJECT, "GXmlSerializableDouble", &g_define_type_info, 0);
		g_type_add_interface_static (gxml_serializable_double_type_id, GXML_TYPE_SERIALIZABLE_PROPERTY, &gxml_serializable_property_info);
		g_once_init_leave (&gxml_serializable_double_type_id__volatile, gxml_serializable_double_type_id);
	}
	return gxml_serializable_double_type_id__volatile;
}



